from datetime import datetime
from typing import Tuple
from uuid import UUID

from pydantic import BaseModel, ValidationError, field_validator
from pydantic_core.core_schema import FieldValidationInfo

class BaseUser(BaseModel):
    class Config:
        # from pydantic.config import BaseConfig
        frozen = True

    uuid: UUID
    id: int
    phone: str

    is_active: bool = False
    is_staff: bool = False
    email_active: bool = False
    phone_active: bool = False

    created_at: datetime
    updated_at: datetime
    last_login: datetime

    permissions: Tuple[str, ...] = tuple
    groups: Tuple[str, ...] = tuple

    position: int | None = None
    firstname: str | None = ''
    lastname: str | None = ''
    avatar: str | None = ''
    email: str | None = ''

    is_superuser: bool = False

    @field_validator('position')
    def validate_position(cls, val: int, info: FieldValidationInfo) -> int:
        #  в качестве работающего примера валидации
        if (val and val < 1) or val == 0:
            raise ValueError('position must be greater then 0')
        return val

    @property
    def is_authenticated(self):
        """
        Always return True. This is a way to tell if the user has been
        authenticated in templates.
        """
        return True

    @property
    def TestP(self):
        return 'TestP'

    def has_perm(self, perm):
        """
        Return True if the user has the specified permission.
        """
        # Active superusers have all permissions.
        if self.is_active and self.is_superuser:
            return True

        # Otherwise we need to check the backends.
        return perm in self.permissions

    def has_perms(self, perm_list):
        """
        Return True if the user has each of the specified permissions.
        """
        set_perm_list = set(perm_list)
        return set(self.permissions).intersection(set_perm_list) == set_perm_list



class AdminUser(BaseUser):
    pass


__all__ = ['AdminUser', 'BaseUser']




import http
import typing
import warnings

__all__ = ("HTTPException", "WebSocketException")


class HTTPException(Exception):
    def __init__(
        self,
        status_code: int,
        detail: typing.Optional[str] = None,
        headers: typing.Optional[dict] = None,
    ) -> None:
        if detail is None:
            detail = http.HTTPStatus(status_code).phrase
        self.status_code = status_code
        self.detail = detail
        self.headers = headers

    def __repr__(self) -> str:
        class_name = self.__class__.__name__
        return f"{class_name}(status_code={self.status_code!r}, detail={self.detail!r})"

class WebSocketException(Exception):
    def __init__(self, code: int, reason: typing.Optional[str] = None) -> None:
        self.code = code
        self.reason = reason or ""

    def __repr__(self) -> str:
        class_name = self.__class__.__name__
        return f"{class_name}(code={self.code!r}, reason={self.reason!r})"



class RuntimeError(Exception):
    """ Unspecified run-time error. """
    def __init__(self, *args, **kwargs): # real signature unknown
        pass

    @staticmethod # known case of __new__
    def __new__(*args, **kwargs): # real signature unknown
        """ Create and return a new object.  See help(type) for accurate signature. """
        pass


class NotImplementedError(RuntimeError):
    """ Method or function hasn't been implemented yet. """
    def __init__(self, *args, **kwargs): # real signature unknown
        pass

    @staticmethod # known case of __new__
    def __new__(*args, **kwargs): # real signature unknown
        """ Create and return a new object.  See help(type) for accurate signature. """
        pass


from dataclasses import asdict, dataclass, field
from datetime import datetime
from uuid import UUID

from pydantic import BaseModel

from ..constants import Role


class Advertiser(BaseModel):
    class Config:
        # pydantic.config.BaseConfig
        frozen = True

    uuid: UUID
    id: int
    profile_id: int
    profile_uuid: UUID


    moderate: int
    premoderate_booking: bool

    created_at: datetime
    updated_at: datetime

    company_name: str | None = ''
    name: str | None = ''
    email: str | None = ''
    phone: str | None = ''
    whatsapp: str | None = ''
    telegram: str | None = ''
    note: str | None = ''
    weblink: str | None = ''
    instagram: str | None = ''

    balance_reach: int | None = None
    balance_reach_all: int | None = None
    description: str | None = ''
    group_payer_id: int | None = None


class Category(BaseModel):
    class Config:
        frozen = True

    id: int
    type: int
    name: str
    parent_id: int | None = None



class Instagram(BaseModel):
    class Config:
        frozen = True

    uuid: UUID
    id: int
    profile_id: int
    profile_uuid: UUID

    created_at: datetime
    updated_at: datetime

    is_private: bool = False
    is_verified: bool = False
    checked_at: datetime | None = None


    reach: int | None = None
    post_reach: int | None = None
    booking_limit: int | None = None
    picture: str | None = ''
    pkid: str | None = ''
    categories: list[Category] = []

    count_media: int | None = 0
    full_name: str | None = ''
    external_url: str | None = ''


class TestM(BaseModel):
    class Config:
        frozen = True

    id: int
    categories: list[Category] = []

class AnonymousProfile(BaseModel):
    class Config:
        frozen = True

    uuid: UUID = None
    user_id: int | None = None
    role: int = Role.ANONYMOUS
    advertiser: Advertiser | None = None
    instagram: Instagram | None = None

    def __str__(self):
        return "AnonymousProfile"

class Profile(BaseModel):
    class Config:
        frozen = True

    uuid: UUID
    user_id: int

    role: int

    advertiser: Advertiser | None = None
    instagram: Instagram | None = None

    def __post_init__(self):
        if self.role == Role.ADVERTISER and not self.advertiser:
            raise ValueError('Advertiser must be set')
        elif self.role == Role.INSTAGRAM and not self.instagram:
            raise ValueError('Instagram must be set')


__all__ = [
    'Profile', 'AnonymousProfile',
]


from pydantic_settings import BaseSettings


class TokenSettings(BaseSettings):
    URL: str | None = None #'redis://localhost:6378/1'
    HOST: str = '127.0.0.1'
    PORT: int = 6379
    PASSWORD: str | None = None
    DB: int = 0
    ACCESS_PREFIX: str = 'access'
    REFRESH_PREFIX: str = 'refresh'
    TOTP_PREFIX: str = 'totp'
    PROFILE_PREFIX: str = 'profile'
    TOKEN_STORAGE: str = 'headers'

    class Config:
        env_prefix = 'REDIS_AUTH_'


app_settings = TokenSettings()

__all__ = [
    'app_settings',
]





class Role:
    ADVERTISER = -1
    INSTAGRAM = 1
    PARTNER = 100
    GROUP_PAYER = 200

    ANONYMOUS = 900

    CHOICES = [
        (INSTAGRAM, 'Instagram'),
        (ADVERTISER, 'Advertiser'),
        (PARTNER, 'Partner'),
        (GROUP_PAYER, 'Group payer')
    ]

class BloggerCategory:
    FOOD = 0
    AUTO = 1
    ANIMALS = 2
    ENTERTAINMENT = 3
    RECREATION = 4
    CLOTHES = 5
    BEAUTY = 6
    SPORT = 7
    CHILD = 8
    TRAVEL = 9
    OTHER = 100

    CHOICES = [
        (FOOD, 'Food'),
        (AUTO, 'Auto'),
        (ANIMALS, 'Animals'),
        (ENTERTAINMENT, 'Entertainment'),
        (RECREATION, 'Recreation'),
        (CLOTHES, 'Clothes'),
        (BEAUTY, 'Beauty'),
        (SPORT, 'Sport'),
        (CHILD, 'Child'),
        (TRAVEL, 'Travel'),
        (OTHER, 'Other'),
    ]



class STAFF_GROUPS:
    STAFF = 'staff'
    TOP_MANAGERS = "top_managers"
    BRANCH_ADVERTISERS = "branch_advertisers"
    BRANCH_BLOGGERS = "branch_bloggers"
    BRANCH_BOOKINGS = "branch_bookings"

    CHOICES_RU = (
        (STAFF, 'Сотрудники'),
        (TOP_MANAGERS, 'Топ менеджеры'),
        (BRANCH_ADVERTISERS, 'Сопровождение рекломодателей'),
        (BRANCH_BLOGGERS, 'Сопровождение блогеров'),
        (BRANCH_BOOKINGS, 'Сопровождение сделок'),
    )

class PERMS:
    USER_VIEW_ALL = "user_view_all"
    USER_NAME_AVATAR_EDIT = "user_name_avatar_edit"
    USER_SET_GROUP = "user_set_group"
    USER_SET_PERMS = "user_set_perms"
    USER_DATA_DWD = "user_data_dwd"

    ADV_VIEW_ALL = "adv_view_all"
    ADV_VIEW_ASSINED = "adv_view_assined"
    ADV_ADD = "adv_add"
    ADV_EDIT = "adv_edit"
    ADV_DELETE = "adv_delete"
    ADV_DATA_DWD = "adv_data_dwd"

    OFFER_VIEW_ALL = "offer_view_all"
    OFFER_VIEW_ASSINED = "offer_view_assined"
    OFFER_ADD = "offer_add"
    OFFER_EDIT_ALL = "offer_edit_all"
    OFFER_EDIT_ASSINED = "offer_edit_assined"
    OFFER_DELETE = "offer_delete"
    OFFER_DATA_DWD = "offer_data_dwd"

    BLOGGER_VIEW_ALL = "blogger_view_all"
    BLOGGER_VIEW_ASSINED = "blogger_view_assined"
    BLOGGER_EDIT_ALL = "blogger_edit_all"
    BLOGGER_EDIT_ASSINED = "blogger_edit_assined"
    BLOGGER_DATA_DWD = "blogger_data_dwd"

    BOOK_VIEW_ALL = "book_view_all"
    BOOK_VIEW_ASSINED = "book_view_assined"
    BOOK_EDIT_ALL = "book_edit_all"
    BOOK_EDIT_ASSINED = "book_edit_assined"
    BOOK_DELETE = "book_delete"
    BOOK_MODERATE_DONE_ALL = "book_moderate_done_all"
    BOOK_MODERATE_DONE_ASSINED = "book_moderate_done_assined"
    BOOK_DATA_DWD = "book_data_dwd"

    CHOICES_RU = (
        (USER_VIEW_ALL, 'Просмотр пользователей'),
        (USER_NAME_AVATAR_EDIT, 'Рерактирование имени/аватара'),
        (USER_SET_GROUP, 'Добавление в группу'),
        (USER_SET_PERMS, 'Назначение прав'),
        (USER_DATA_DWD, 'Скачивание'),

        (ADV_VIEW_ALL, 'Просмотр всех рекламодателей'),
        (ADV_VIEW_ASSINED, 'Просмотр своих рекламодателей'),
        (ADV_ADD, 'Добавление'),
        (ADV_EDIT, 'Редактирование'),
        (ADV_DELETE, 'Удаление (если возможно)'),
        (ADV_DATA_DWD, 'Скачивание данных'),

        (OFFER_VIEW_ALL, 'Просмотр всех товаров'),
        (OFFER_VIEW_ASSINED, 'Просмотр товаров своих рекламодателей'),
        (OFFER_ADD, 'Добавление'),
        (OFFER_EDIT_ALL, 'Редактирование'),
        (OFFER_EDIT_ASSINED, 'Редактирование своих'),
        (OFFER_DELETE, 'Удаление (если возможно)'),
        (OFFER_DATA_DWD, 'Скачивание данных'),

        (BLOGGER_VIEW_ALL, 'Просмотр блогеров'),
        (BLOGGER_VIEW_ASSINED, 'Просмотр своих блогеров'),
        (BLOGGER_EDIT_ALL, 'Редактирование (частичное)'),
        (BLOGGER_EDIT_ASSINED, 'Редактирование своих (частичное)'),
        (BLOGGER_DATA_DWD, 'Скачивание'),

        (BOOK_VIEW_ALL, 'Просмотр'),
        (BOOK_VIEW_ASSINED, 'Other'),
        (BOOK_EDIT_ALL, 'Редактирование всех (частичное)'),
        (BOOK_EDIT_ASSINED, 'Редактирование своих (частичное)'),
        (BOOK_DELETE, 'Other'),
        (BOOK_MODERATE_DONE_ALL, 'Модерация завершения всех сделок'),
        (BOOK_MODERATE_DONE_ASSINED, 'Модерация завершения своих сделок'),
        (BOOK_DATA_DWD, 'Скачивание'),
    )


__all__ = ['PERMS']


import json
import os

import redis

from .models import *
from .settings import app_settings

DEFAULT_TTL = 300
DEFAULT_ACCESS_TTL = 259200

class RedisClient:
    _prefix = "basic"
    _ttl = DEFAULT_TTL

    def __init__(self, host: str or tuple, port: int or tuple, db: int, password: str):
        host = host if type(host) is not tuple else host[0]
        port = port if type(port) is not tuple else port[0]
        self.__redis_client = redis.StrictRedis(host=host, port=port, db=db, password=password)

    @property
    def conn(self):
        return self.__redis_client

    def __get(self, name: str) -> str | None:
        value = self.conn.get(name)
        return value

    def __put(self, name: str, value: str, ttl: int) -> None:
        '''

        :param name:  redis key
        :param value: json string
        :param ttl:
        :return:
        '''
        if type(value) is dict:
            value_data = json.dumps(value)
        else:
            value_data = value
        self.conn.set(name=name, value=value_data, ex=ttl)

    def is_token_exist(self, token: str, prefix: bool = True) -> str | None:
        key = f"{self._prefix}:{token}" if prefix else token
        return self.__get(key)

    def add_token(self, token: str, json_data: str = None, ttl: int = None, prefix: bool = True):
        key = f"{self._prefix}:{token}" if prefix else token
        ttl = ttl if ttl is not None else self._ttl
        value = json_data if json_data is not None else json.dumps({"token": key})
        self.__put(name=key, value=value, ttl=ttl)


class RedisAccessClient(RedisClient):
    _prefix = app_settings.ACCESS_PREFIX
    _ttl = int(os.getenv("ACCESS_TOKEN_LIFETIME", DEFAULT_ACCESS_TTL))

    def __init__(self,
                 host: str = app_settings.HOST,
                 port: int = app_settings.PORT,
                 db: int = app_settings.DB,
                 password: str = app_settings.PASSWORD):
        super().__init__(host=host, port=port, db=db, password=password)

    def get_user(self, token: str, prefix: bool = True) -> BaseUser | None:
        user_json = self.is_token_exist(token, prefix=prefix)
        if user_json:
            return BaseUser.parse_raw(user_json)
        return None


class RedisRefreshClient(RedisClient):
    _prefix = app_settings.REFRESH_PREFIX
    _ttl = int(os.getenv("REFRESH_TOKEN_LIFETIME", DEFAULT_TTL))

    def __init__(self,
                 host: str = app_settings.HOST,
                 port: int = app_settings.PORT,
                 db: int = app_settings.DB,
                 password: str = app_settings.PASSWORD):
        super().__init__(host=host, port=port, db=db, password=password)


class RedisTotpClient(RedisClient):
    _prefix = app_settings.TOTP_PREFIX

    def __init__(self, host: str, port: int, db: int, password: str):
        super().__init__(host=host, port=port, db=db, password=password)

class RedisProfileClient(RedisClient):
    _prefix = app_settings.PROFILE_PREFIX
    _ttl = None

    def __init__(self,
                 host: str = app_settings.HOST,
                 port: int = app_settings.PORT,
                 db: int = app_settings.DB,
                 password: str = app_settings.PASSWORD):
        super().__init__(host=host, port=port, db=db, password=password)

    def is_profile_exist(self, uuid: str, prefix: bool = True) -> str | None:
        return super(RedisProfileClient, self).is_token_exist(uuid, prefix=prefix)

    def get_profile(self, uuid: str, prefix: bool = True) -> Profile | AnonymousProfile:
        profile_json = self.is_profile_exist(uuid, prefix=prefix)
        if profile_json:
            return Profile.parse_raw(profile_json)
        return AnonymousProfile()

    def is_token_exist(self, *args, **kwargs):
        raise NotImplementedError("Not declared here.")




from .exceptions import HTTPException
from .providers import RedisAccessClient

# Header encoding (see RFC5987)
HTTP_HEADER_ENCODING = 'iso-8859-1'


def get_authorization_header(request):
    """
    Return request's 'Authorization:' header, as a bytestring.
    Hide some test client ickyness where the header can be unicode.
    """
    auth = request.META.get('HTTP_AUTHORIZATION', b'')
    # auth = request.headers.get('Authorization', b'')
    if isinstance(auth, str):
        # Work around django test client oddness
        auth = auth.encode(HTTP_HEADER_ENCODING)
    return auth

def get_token_from_cookies(request) -> str:
    # cookie = request.headers.get('Cookie', '')
    cookie = request.META.get('HTTP_COOKIE', '')
    token = None
    try:
        cookie_dict = dict([val.split('=') for val in cookie.split()])
        token = cookie_dict.get('auth_token')
    except:
        pass
    return token

def get_token_from_header(request) -> str:
    auth = get_authorization_header(request).split()
    token = None
    if len(auth) == 2:
        try:
            token = auth[1].decode()
        except UnicodeError:
            token = None
    return token

class BaseAuthentication:
    def authenticate(self, request):
        """
        Authenticate the request and return a two-tuple of (user, token).
        """
        raise NotImplementedError(".authenticate() must be overridden.")

    def authenticate_header(self, request):
        pass

class ApiTokenRedisAuthentication(BaseAuthentication):
    """
    Simple token based authentication.
    Clients should authenticate by passing the token key in the "Authorization"
    HTTP header, prepended with the string "Token ".  For example:
        Authorization: Token 401f7ac837da42b97f613d789819ff93537bee6a
    """
    keyword = 'Token'
    def authenticate(self, request):
        auth = get_authorization_header(request).split()
        if not auth or auth[0].lower() != self.keyword.lower().encode():
            return None

        if len(auth) == 1:
            msg = 'Invalid token header. No credentials provided.'
            raise HTTPException(status_code=401, detail=msg)
        elif len(auth) > 2:
            msg = 'Invalid token header. Token string should not contain spaces.'
            raise HTTPException(status_code=401, detail=msg)
        try:
            token = auth[1].decode()
        except UnicodeError:
            msg = 'Invalid token header. Token string should not contain invalid characters.'
            raise HTTPException(status_code=401, detail=msg)
        return self.authenticate_credentials(token)

    def authenticate_credentials(self, key):
        token_service = RedisAccessClient()
        user = token_service.get_user(key)
        if not user:
            raise HTTPException(status_code=401, detail='Invalid token.')

        if not user.is_active:
            raise HTTPException(status_code=401, detail='User inactive or deleted.')

        return (user, key)

    def authenticate_header(self, request):
        return self.keyword


__all__ = [
    'ApiTokenRedisAuthentication',
    'get_token_from_cookies',
    'get_token_from_header',
]





import os

from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()


# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


setup(
    name='barter-auth',
    version='0.1.6',
    include_package_data=True,
    license='BSD',
    description='Django bulk admin enables you to bulk add, bulk edit, bulk upload and bulk select in django admin.',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://github.com/kabilovtoha/barter_auth',
    author='akatoha',
    author_email='kabilov2011@gmail.com',
    install_requires=[
        'Django>=3.2',
        'six>=1.15.0',
        'pydantic-settings>=2.1.0'
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',

        'Programming Language :: Python :: 3.11',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)